@@ -14,6 +14,7 @@
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 #include <scsi/sg.h>
 #include <scsi/scsi.h>
@@ -27,9 +28,9 @@
 #include <string>
 #include <map>
 
-__ID("@(#) $Id: scsi.cc 2433 2012-01-10 22:01:30Z lyonel $");
+__ID("@(#) $Id$");
 
-#define SG_X "/dev/sg%d"
+#define SG_X "/dev/sg*"
 #define SG_MAJOR 21
 
 #ifndef SCSI_IOCTL_GET_PCI
@@ -195,10 +196,10 @@ static const char *devices[] =
   "/dev/sr*",
   "/dev/cd*",
   "/dev/dvd*",
-  "/dev/st*",
   "/dev/nst*",
   "/dev/nosst*",
-  "/dev/tape*",
+  "/dev/ntape*",
+  "/dev/nrtape*",
   NULL
 };
 
@@ -439,7 +440,7 @@ static u_int16_t decode_word(void *ptr)
 static bool do_inquiry(int sg_fd,
 hwNode & node)
 {
-  char rsp_buff[MX_ALLOC_LEN + 1];
+  uint8_t rsp_buff[MX_ALLOC_LEN + 1];
   int k;
   unsigned int len;
 
@@ -452,7 +453,7 @@ hwNode & node)
 
   len = (unsigned int) rsp_buff[4] + 5;
 
-  if ((len > 36) && (len < 256))
+  if ((len >= 36) && (len < 256))
   {
     memset(rsp_buff, 0, sizeof(rsp_buff));
     if (!do_inq(sg_fd, 0, 0, 0, rsp_buff, len, 1))
@@ -469,11 +470,11 @@ hwNode & node)
   if (rsp_buff[1] & 0x80)
     node.addCapability("removable", "support is removable");
 
-  node.setVendor(string(rsp_buff + 8, 8));
+  node.setVendor(string((char *)rsp_buff + 8, 8));
   if (len > 16)
-    node.setProduct(string(rsp_buff + 16, 16));
+    node.setProduct(string((char *)rsp_buff + 16, 16));
   if (len > 32)
-    node.setVersion(string(rsp_buff + 32, 4));
+    node.setVersion(string((char *)rsp_buff + 32, 4));
 
   if (ansiversion)
     node.setConfig("ansiversion", tostring(ansiversion));
@@ -481,9 +482,9 @@ hwNode & node)
   memset(rsp_buff, 0, sizeof(rsp_buff));
   if (do_inq(sg_fd, 0, 1, 0x80, rsp_buff, MX_ALLOC_LEN, 0))
   {
-    len = rsp_buff[3];
-    if (len > 0)
-      node.setSerial(hw::strip(string(rsp_buff + 4, len)));
+    uint8_t _len = rsp_buff[3];
+    if (_len > 0)
+      node.setSerial(hw::strip(string((char *)rsp_buff + 4, _len)));
   }
 
   memset(rsp_buff, 0, sizeof(rsp_buff));
@@ -494,14 +495,14 @@ hwNode & node)
     unsigned long long cyl = 0;
     unsigned long long sectors = 0;
     unsigned long rpm = 0;
-    u_int8_t *end = (u_int8_t *) rsp_buff + (u_int8_t) rsp_buff[0];
-    u_int8_t *p = NULL;
+    uint8_t *end = rsp_buff + rsp_buff[0];
+    uint8_t *p = NULL;
 
     if (rsp_buff[3] == 8)
       sectsize = decode_3_bytes(rsp_buff + 9);
 
-    p = (u_int8_t *) & rsp_buff[4];
-    p += (u_int8_t) rsp_buff[3];
+    p = & rsp_buff[4];
+    p += rsp_buff[3];
     while (p < end)
     {
       u_int8_t page = *p & 0x3F;
@@ -653,10 +654,9 @@ static bool atapi(const hwNode & n)
 }
 
 
-static bool scan_sg(int sg,
-hwNode & n)
+static void scan_sg(hwNode & n)
 {
-  char buffer[20];
+  int sg;
   int fd = -1;
   My_sg_scsi_id m_id;
   char slot_name[64];                             // should be 16 but some 2.6 kernels require 32 bytes
@@ -665,22 +665,30 @@ hwNode & n)
   hwNode *parent = NULL;
   int emulated = 0;
   bool ghostdeventry = false;
+  size_t j;
+  glob_t entries;
 
-  snprintf(buffer, sizeof(buffer), SG_X, sg);
+  if(glob(SG_X, 0, NULL, &entries) == 0)
+  {
+    for(j=0; j < entries.gl_pathc; j++)
+    {
+      sg = strtol(strpbrk(entries.gl_pathv[j], "0123456789"), NULL, 10);
 
-  ghostdeventry = !exists(buffer);
+      ghostdeventry = !exists(entries.gl_pathv[j]);
 
-  if(ghostdeventry) mknod(buffer, (S_IFCHR | S_IREAD), MKDEV(SG_MAJOR, sg));
-  fd = open(buffer, OPEN_FLAG | O_NONBLOCK);
-  if(ghostdeventry) unlink(buffer);
+      if(ghostdeventry)
+        mknod(entries.gl_pathv[j], (S_IFCHR | S_IREAD), MKDEV(SG_MAJOR, sg));
+      fd = open(entries.gl_pathv[j], OPEN_FLAG | O_NONBLOCK);
+      if(ghostdeventry)
+        unlink(entries.gl_pathv[j]);
   if (fd < 0)
-    return false;
+        continue;
 
   memset(&m_id, 0, sizeof(m_id));
   if (ioctl(fd, SG_GET_SCSI_ID, &m_id) < 0)
   {
     close(fd);
-    return true;                                  // we failed to get info but still hope we can continue
+        continue;        // we failed to get info but still hope we can continue
   }
 
   emulated = 0;
@@ -773,11 +781,8 @@ hwNode & n)
   if (!parent)
     parent = n.addChild(hwNode("scsi", hw::storage));
 
-  if (!parent)
+  if (parent)
   {
-    close(fd);
-    return true;
-  }
 
   if(parent->getBusInfo() == "")
     parent->setBusInfo(guessBusInfo(hw::strip(slot_name)));
@@ -789,10 +794,12 @@ hwNode & n)
     parent->addCapability("emulated", "Emulated device");
   }
   parent->addChild(device);
+  }
 
   close(fd);
-
-  return true;
+    }
+    globfree(&entries);
+  }
 }
 
 
@@ -835,7 +842,7 @@ static bool scan_hosts(hwNode & node)
 
           if (!controller)
           {
-            string parentbusinfo = sysfs_getbusinfo(sysfs::entry::byClass("scsi_host", host_kname(number)));
+            string parentbusinfo = sysfs::entry::byClass("scsi_host", host_kname(number)).businfo();
 
             controller = node.findChildByBusInfo(parentbusinfo);
           }
@@ -889,12 +896,9 @@ static bool scan_hosts(hwNode & node)
 
 bool scan_scsi(hwNode & n)
 {
-  int i = 0;
-
   scan_devices();
 
-  while (scan_sg(i, n))
-    i++;
+  scan_sg(n);
 
   scan_hosts(n);
 
