@@ -3,7 +3,19 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
-static char *id = "@(#) $Id: stock.c 2336 2010-10-05 09:34:27Z lyonel $";
+static char *id = "@(#) $Id$";
+
+#define UIFILE "gtk-lshw.ui"
+
+GtkWidget *mainwindow = NULL;
+GtkWidget *about = NULL;
+GtkWidget *list1 = NULL;
+GtkWidget *list2 = NULL;
+GtkWidget *list3 = NULL;
+GtkWidget *description = NULL;
+GtkWidget *go_up_button = NULL;
+GtkWidget *save_button = NULL;
+GtkWidget *statusbar = NULL;
 
 static struct StockIcon
 {
@@ -48,7 +60,7 @@ static struct StockIcon
 };
 
 static gchar *
-find_file(const char *base)
+find_file(const char *base, const char *dir)
 {
   char *filename;
   char *basedir;
@@ -57,9 +69,9 @@ find_file(const char *base)
     return NULL;
 
   if((basedir = getenv("BASEDIR")))
-    filename = g_build_filename(basedir, "artwork", base, NULL);
+    filename = g_build_filename(basedir, dir, base, NULL);
   else
-    filename = g_build_filename(DATADIR, "lshw", "artwork", base, NULL);
+    filename = g_build_filename(DATADIR, "lshw", dir, base, NULL);
 
   if (!g_file_test(filename, G_FILE_TEST_EXISTS))
   {
@@ -99,7 +111,7 @@ lshw_gtk_stock_init(void)
     GtkIconSet *iconset;
     gchar *filename;
 
-      filename = find_file(stock_icons[i].filename);
+      filename = find_file(stock_icons[i].filename, "artwork");
 
       if (filename == NULL)
         continue;
@@ -125,3 +137,51 @@ lshw_gtk_stock_init(void)
 
   (void) &id;                                     /* avoid "id defined but not used" warning */
 }
+
+void lshw_ui_init(void)
+{
+  GError *error = NULL;
+  GtkBuilder *builder = NULL;
+  GdkPixbuf *icon = NULL;
+  gchar *uiname = NULL;
+
+  mainwindow = NULL;
+
+  builder = gtk_builder_new();
+  uiname = find_file(UIFILE, "ui");
+  if(!uiname)
+  {
+    g_critical( "Could not find UI file: %s", UIFILE );
+    return;
+  }
+  if(!gtk_builder_add_from_file(builder, uiname, &error))
+  {
+    g_critical( "Could not create UI: %s", error->message );
+    g_free( error );
+    g_object_unref( G_OBJECT( builder ) );
+    return;
+  }
+  g_free(uiname);
+
+  mainwindow = GTK_WIDGET( gtk_builder_get_object( builder, "mainwindow" ) );
+  about = GTK_WIDGET( gtk_builder_get_object( builder, "aboutlshw" ) );
+  list1 = GTK_WIDGET(gtk_builder_get_object( builder, "treeview1"));
+  list2 = GTK_WIDGET(gtk_builder_get_object( builder, "treeview2"));
+  list3 = GTK_WIDGET(gtk_builder_get_object( builder, "treeview3"));
+  description = GTK_WIDGET(gtk_builder_get_object( builder, "description"));
+  go_up_button = GTK_WIDGET(gtk_builder_get_object( builder, "upbutton"));
+  save_button = GTK_WIDGET(gtk_builder_get_object( builder, "savebutton"));
+  statusbar = GTK_WIDGET(gtk_builder_get_object( builder, "statusbar"));
+  gtk_builder_connect_signals( builder, mainwindow );
+  g_object_unref( G_OBJECT( builder ) );
+
+  icon = gtk_widget_render_icon(GTK_WIDGET(mainwindow),
+    "lshw-logo",
+    GTK_ICON_SIZE_DIALOG,
+    NULL);
+  if(GDK_IS_PIXBUF(icon))
+  {
+    gtk_window_set_icon(GTK_WINDOW(mainwindow), icon);
+    gtk_window_set_default_icon(icon);
+  }
+}
